﻿using Sdl.Core.Globalization;
using Sdl.FileTypeSupport.Framework;
using Sdl.FileTypeSupport.Framework.IntegrationApi;
using Sdl.FileTypeSupport.Framework.NativeApi;

namespace Versacom.Sdl.Plugins.Filters.Demo
{
    /// <summary>
    /// "Root" of a FileTypeSupport API plugin, this builds the plugin chain and defines components used
    /// by this plugin.
    /// </summary>
    [FileTypeComponentBuilder(Id = "DemoFilterExtension",
                              Description = "DemoFilter_FilterComponentBuilderExtension_Description",
                              Name = "DemoFilter_FilterComponentBuilderExtension_Name")]
    public class DemoComponentBuilder : IFileTypeComponentBuilder, IFileTypeDefinitionAware
    {
        #region IFileTypeDefinitionAware members
        /// <summary>
        /// Stores the IFileTypeDefinition
        /// </summary>
        public IFileTypeDefinition FileTypeDefinition { get; set; }
        #endregion
        
        #region IFileTypeComponentBuilder members
        /// <summary>
        /// Allows an IFileTypeManager to be set, defaults to the default one provided by SDL
        /// </summary>
        public IFileTypeManager FileTypeManager { get; set; }

        /// <summary>
        /// Returns a FileExtractor that uses a DemoParser as its INativeExtractor
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public IFileExtractor BuildFileExtractor(string name)
        {
            return this.FileTypeManager.BuildFileExtractor(
                this.FileTypeManager.BuildNativeExtractor(
                    (INativeFileParser)new DemoParser()
                ), 
                (IFileTypeComponentBuilder)this
            );
        }

        /// <summary>
        /// Returns a FileGenerator that uses a DemoWriter as its INativeFileWriter
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public IFileGenerator BuildFileGenerator(string name)
        {
            return this.FileTypeManager.BuildFileGenerator(
                    this.FileTypeManager.BuildNativeGenerator((INativeFileWriter)new DemoWriter())
            );
        }

        /// <summary>
        /// Returns a DemoFileSniffer
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public INativeFileSniffer BuildFileSniffer(string name)
        {
            return (INativeFileSniffer)new DemoFileSniffer();
        }

        /// <summary>
        /// Builts the IFileTypeInformation that describes a DemoFile
        /// 
        /// This will be used to fill dialogs and UI information about the file type
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public IFileTypeInformation BuildFileTypeInformation(string name)
        {
            IFileTypeInformation ifileTypeInformation = this.FileTypeManager.BuildFileTypeInformation();
            // show in filetype columns, uniquely identifies the filter in sdl
            ifileTypeInformation.FileTypeDefinitionId = new FileTypeDefinitionId("Demo Filter v 1.0.0.0");
            // Used in the FileTypes tree, and everywhere where a FileType is displayed, except BrowseDialogs
            ifileTypeInformation.FileTypeName = new LocalizableString(PluginResources.DemoFileTypeName);
            // Used in some File dialogs, such as Save As
            ifileTypeInformation.FileTypeDocumentName = new LocalizableString(PluginResources.DemoFileTypeDocumentName);
            // Used in File dialogs, such as browse to generate a Filter in the dropdown
            ifileTypeInformation.FileTypeDocumentsName = new LocalizableString(PluginResources.DemoFileTypeDocumentsName);
            // Displayed in a huge textblock when selected in the File types tree
            ifileTypeInformation.Description = new LocalizableString(PluginResources.DemoFileTypeDescription);
            ifileTypeInformation.FileDialogWildcardExpression = "*.demo";
            // I guess its default save as extension
            ifileTypeInformation.DefaultFileExtension = "demo";
            ifileTypeInformation.Icon = new IconDescriptor(PluginResources.DemoFileTypeIcon);

            ifileTypeInformation.WinFormSettingsPageIds =
                new string[2]
                {
                    "DemoFilter_GeneralSettings",
                    "DemoFilter_EmbeddedContentSettings"
                };
            return ifileTypeInformation;
        }

        /// <summary>
        /// Returns the QuickTagsFactory of the FileTypeManager 
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public IQuickTagsFactory BuildQuickTagsFactory(string name)
        {
            return this.FileTypeManager.BuildQuickTagsFactory();
        }

        /// <summary>
        /// Returns null
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public IAbstractPreviewApplication BuildPreviewApplication(string name)
        {
            return (IAbstractPreviewApplication)null;
        }

        /// <summary>
        /// Returns null
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public IAbstractPreviewControl BuildPreviewControl(string name)
        {
            return (IAbstractPreviewControl)null;
        }

        /// <summary>
        /// Returns a PreviewSetsFactory from the FileTypeManager
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public IPreviewSetsFactory BuildPreviewSetsFactory(string name)
        {
            return this.FileTypeManager.BuildPreviewSetsFactory();
        }

        /// <summary>
        /// Returns null
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public IAbstractGenerator BuildAbstractGenerator(string name)
        {
            return (IAbstractGenerator)null;
        }

        /// <summary>
        /// Returns null
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public IAdditionalGeneratorsInfo BuildAdditionalGeneratorsInfo(string name)
        {
            return (IAdditionalGeneratorsInfo)null;
        }

        /// <summary>
        /// Returns null
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public IBilingualDocumentGenerator BuildBilingualGenerator(string name)
        {
            return (IBilingualDocumentGenerator)null;
        }


        /// <summary>
        /// Returns null
        /// </summary>
        /// <param name="name"></param>
        /// <returns></returns>
        public IVerifierCollection BuildVerifierCollection(string name)
        {
            return (IVerifierCollection)null;
        }
        #endregion
    }
}
